@@ -51,9 +51,33 @@ impl Item {
 // per CLI invocation, so there isn't a huge need for stack optimization yet.
 pub type Stack = Vec<Item>;
 
+pub enum Backend {
+    HomeDir,
+}
+
+impl Backend {
+    pub fn load(&self, stack_name: &str) -> Result<Stack, impl Error> {
+        match self {
+            Backend::HomeDir => load_from_homedir(stack_name),
+        }
+    }
+
+    pub fn save(&self, stack_name: &str, items: Stack) -> Result<(), impl Error> {
+        match self {
+            Backend::HomeDir => save_to_homedir(stack_name, items),
+        }
+    }
+
+    pub fn list_stacks(&self) -> Result<Vec<String>, impl Error> {
+        match self {
+            Backend::HomeDir => list_stacks_from_homedir(),
+        }
+    }
+}
+
 /// Save a stack of items.
 // TODO: Create a custom error. This is returning raw filesystem errors.
-pub fn save(stack_name: &str, items: Stack) -> Result<(), impl Error> {
+fn save_to_homedir(stack_name: &str, items: Stack) -> Result<(), impl Error> {
     let data_path: String = sigi_file(stack_name);
     let json: String = serde_json::to_string(&items).unwrap();
     let result = fs::write(&data_path, &json);
@@ -67,7 +91,7 @@ pub fn save(stack_name: &str, items: Stack) -> Result<(), impl Error> {
 
 /// Load a stack of items.
 // TODO: Create a custom error. This is returning raw serialization errors.
-pub fn load(stack_name: &str) -> Result<Stack, impl Error> {
+fn load_from_homedir(stack_name: &str) -> Result<Stack, impl Error> {
     let data_path: String = sigi_file(stack_name);
     let read_result = fs::read_to_string(data_path);
     if read_result.is_err() && read_result.as_ref().unwrap_err().kind() == ErrorKind::NotFound {
@@ -87,6 +111,17 @@ pub fn load(stack_name: &str) -> Result<Stack, impl Error> {
     result
 }
 
+fn list_stacks_from_homedir() -> Result<Vec<String>, impl Error> {
+    let dot_json = ".json";
+    fs::read_dir(sigi_path()).map(|files| {
+        files
+            .map(|file| file.unwrap().file_name().into_string().unwrap())
+            .filter(|filename| filename.ends_with(dot_json))
+            .map(|filename| filename.strip_suffix(dot_json).unwrap().to_string())
+            .collect::<Vec<_>>()
+    })
+}
+
 fn sigi_path() -> String {
     let home = env::var("HOME").or_else(|_| env::var("HOMEDRIVE")).unwrap();
     let path = format!("{}/.local/share/sigi", home);
@@ -100,19 +135,19 @@ fn sigi_file(filename: &str) -> String {
 
 /// A single stack item. Used for backwards compatibility with versions of Sigi v1.
 #[derive(Serialize, Deserialize, Debug, Clone)]
-pub struct V1Item {
-    pub name: String,
-    pub created: DateTime<Local>,
-    pub succeeded: Option<DateTime<Local>>,
-    pub failed: Option<DateTime<Local>>,
+struct V1Item {
+    name: String,
+    created: DateTime<Local>,
+    succeeded: Option<DateTime<Local>>,
+    failed: Option<DateTime<Local>>,
 }
 
 /// A stack of items. Used for backwards compatibility with versions of Sigi v1.
-pub type V1Stack = Vec<V1Item>;
+type V1Stack = Vec<V1Item>;
 
 /// Attempt to read a V1 format file.
 fn v1_load(json_blob: &str) -> Result<V1Stack, impl Error> {
-    serde_json::from_str(&json_blob)
+    serde_json::from_str(json_blob)
 }
 
 fn v1_to_modern(v1stack: V1Stack) -> Stack {
@@ -126,7 +161,7 @@ fn v1_to_modern(v1stack: V1Stack) -> Stack {
                 v1item.failed.map(|dt| ("deleted", dt)),
             ]
             .into_iter()
-            .filter_map(|entry| entry)
+            .flatten()
             .map(|(s, dt)| (s.to_string(), dt))
             .collect();
             history.sort_by_key(|(_, dt)| *dt);
